@@ -1,17 +1,17 @@
 #
-# $Id: FreeBSD.pm,v efe6729c89ce 2015/01/31 17:37:12 gomor $
+# $Id: FreeBSD.pm,v 788e55306393 2015/02/04 16:15:59 gomor $
 #
 package Net::Routing::FreeBSD;
 use strict;
 use warnings;
 
-our $VERSION = '0.20';
+our $VERSION = '0.30';
 
 use base qw(Net::Routing::Linux);
 
 use Net::IPv4Addr;
 use Net::IPv6Addr;
-use Net::Routing qw($Error NR_DEFAULT_ROUTE4 NR_DEFAULT_ROUTE6 NR_LOCAL_ROUTE4 NR_LOCAL_ROUTE6);
+use Net::Routing qw($Error :constants);
 
 sub get {
    my $self = shift;
@@ -34,19 +34,57 @@ sub get {
    return $self->SUPER::get($cmd4, $cmd6);
 }
 
-sub _get_inet {
+sub _get_inet4 {
    my $self = shift;
    my ($lines) = @_;
 
    my @routes = ();
+   my %cache = ();
+
+   # FreeBSD 9.x
+   # Destination        Gateway            Flags    Refs      Use  Netif Expire
+   # default            8.8.210.254        UGS         0 14188719    em0
+   #
+   # FreeBSD 10.x
+   # Destination        Gateway            Flags      Netif Expire
+   # default            8.8.25.254         UGS         re0
+
+   my $freebsd_version = '10.x';
 
    for my $line (@$lines) {
+      # FreeBSD 10.1-RELEASE
+      if ($line =~ /^\s*destination\s+gateway\s+flags\s+netif\s+expire\s*$/i) {
+         #print STDERR "*** DEBUG FreeBSD 10.x\n";
+         $freebsd_version = '10.x';
+         next;
+      }
+      # FreeBSD 9.3-RELEASE
+      elsif ($line =~ /^\s*destination\s+gateway\s+flags\s+refs\s+use\s+netif\s+expire\s*$/i) {
+         #print STDERR "*** DEBUG FreeBSD 9.x\n";
+         $freebsd_version = '9.x';
+         next;
+      }
+
       my @toks = split(/\s+/, $line);
-      my $route = $toks[0];
-      my $gateway = $toks[1];
-      my $flags = $toks[2];
-      my $interface = $toks[3];
-      my $expire = $toks[4];
+
+      my ($route, $gateway, $flags, $refs, $use, $interface, $expire);
+
+      if ($freebsd_version eq '9.x') {
+         $route = $toks[0];
+         $gateway = $toks[1];
+         $flags = $toks[2];
+         $refs = $toks[3];
+         $use = $toks[4];
+         $interface = $toks[5];
+         $expire = $toks[6];
+      }
+      else {  # Default to FreeBSD 10.x
+         $route = $toks[0];
+         $gateway = $toks[1];
+         $flags = $toks[2];
+         $interface = $toks[3];
+         $expire = $toks[4];
+      }
 
       if (defined($route) && defined($gateway) && defined($interface)) {
          #print STDERR "*** DEBUG $route $gateway $interface\n";
@@ -60,7 +98,7 @@ sub _get_inet {
          }
 
          # A first sanity check to help Net::IPv4Addr
-         if ($route !~ /^[0-9\.]+$/ || $gateway !~ /^[0-9\.]+$/) {
+         if ($gateway !~ m{^[0-9\.]+$} || $route !~ m{^[0-9\.]+(?:/\d+)?$}) {
             next;
          }
 
@@ -97,7 +135,12 @@ sub _get_inet {
             $route{route} .= '/32';
          }
 
-         push @routes, \%route;
+         my $id = $self->_to_psv(\%route);
+         if (! exists($cache{$id})) {
+            #print STDERR "*** DEBUG new $id\n";
+            push @routes, \%route;
+            $cache{$id}++;
+         }
       }
    }
 
@@ -109,9 +152,26 @@ sub _get_inet6 {
    my ($lines) = @_;
 
    my @routes = ();
+   my %cache = ();
+
+   # FreeBSD 9.3-RELEASE
+   # Internet6:
+   # Destination                       Gateway                       Flags      Netif Expire
+   # ::/96                             ::1                           UGRS        lo0 =>
+   # default                           2003:1122:1:ffff:ff:ff:ff:ff  UGS         em0
+   # ::1                               link#5                        UH          lo0
+
+   # FreeBSD 10.1-RELEASE
+   # Internet6:
+   # Destination                       Gateway                       Flags      Netif Expire
+   # ::/96                             ::1                           UGRS        lo0
+   # ::1                               link#2                        UH          lo0
+   # ::ffff:0.0.0.0/96                 ::1                           UGRS        lo0
+   # 2003:1122:2:1a00::/56             link#1                        U           re0
 
    for my $line (@$lines) {
       my @toks = split(/\s+/, $line);
+
       my $route = $toks[0];
       my $gateway = $toks[1];
       my $flag = $toks[2];
@@ -123,18 +183,26 @@ sub _get_inet6 {
          if ($gateway =~ /^link/ || $gateway eq '::1') {
             $gateway = '::';
          }
+         if ($route eq 'default') {
+            $route = '::/0';
+         }
 
          # Special case: an entry with a MAC address means a default gateway
          if ($gateway =~ /^[a-f0-9]{2}:[a-f0-9]{2}:[a-f0-9]{2}:[a-f0-9]{2}:[a-f0-9]{2}:[a-f0-9]{2}$/) {
-            push @routes, {
+            my %route = (
                route => NR_DEFAULT_ROUTE6(),
                gateway => $route,
                interface => $interface,
-            };
+            );
+            my $id = $self->_to_psv(\%route);
+            if (! exists($cache{$id})) {
+               push @routes, \%route;
+               $cache{$id}++;
+            }
          }
 
          # A first sanity check to help Net::IPv6Addr
-         if ($route !~ /^[0-9a-f:\/]+$/i || $gateway !~ /^[0-9a-f:\/]+$/i) {
+         if ($route !~ m{^[0-9a-f:/]+$}i || $gateway !~ m{^[0-9a-f:/]+$}i) {
             next;
          }
 
@@ -159,7 +227,7 @@ sub _get_inet6 {
          );
 
          # Default route
-         if ($route eq '::') {
+         if ($route eq '::/0') {
             $route{default} = 1;
             $route{route} = NR_DEFAULT_ROUTE6();
          }
@@ -174,7 +242,11 @@ sub _get_inet6 {
             $route{route} .= '/128';
          }
 
-         push @routes, \%route;
+         my $id = $self->_to_psv(\%route);
+         if (! exists($cache{$id})) {
+            push @routes, \%route;
+            $cache{$id}++;
+         }
       }
    }
 
